# Learning Object-aware Anchor-free Networks for Real-time Object Tracking (fastOcean TensorRT Version)

- If you have any issue with running our tracker, please feel free to contact zhangzhipeng2017@ia.ac.cn .

## Important Files 
- fastOcean/vot_fastOcean.py <br/>
  The interface between our tracker with VOT2020 python toolkit.
- fastOcean/install.sh <br/>
  The guidance of conda environment and python packages installation. 
- fastOcean/lib <br/>
  Main Code of our tracker.
- fastOcean/experiment <br/>
  Hyperparameters of our tracker.
- fastOcean/snapshot  <br/>
  Pretrained models (empty).


## Running Environment
python3.7 <br/>
pytorch1.1.0 <br/>
cudatoolkit10.0.130 <br/>
Nvidia RTX2080Ti (12GB) <br/>
Intel(R) Xeon(R) Gold 5118 CPU @ 2.30GHz <br/>

## Results (EAO)
| baseline | realtime | 
| ------ | ------ | 
|0.476	| 0.466 | 


## Run Tracker

### Step1. Set up the environment
Before setting up the environment, please install CUDA10.

In the code (fastOcean),
```
bash install.sh $conda_path fastOcean
```
`$conda_path` indicates your anaconda installation path, e.g. `/data/zpzhang/anaconda3`.

### Step2. Install TensorRT
1) install pycuda
```
export C_INCLUDE_PATH=/usr/local/cuda-10.0/include/:${C_INCLUDE_PATH}
export CPLUS_INCLUDE_PATH=/usr/local/cuda-10.0/include/:${CPLUS_INCLUDE_PATH}
pip install pycuda
```
2) download tensorrt
- Go to [NVIDIA-TENSORRT](https://developer.nvidia.com/tensorrt) and then  click `Download Now`.
- Login and download TensorRT7 (please select the version that suits for your platform). We use [TensorRT 7.0.0.11 for Ubuntu 18.04 and CUDA 10.0 tar package](https://developer.nvidia.com/compute/machine-learning/tensorrt/secure/7.0/7.0.0.11/tars/TensorRT-7.0.0.11.Ubuntu-18.04.x86_64-gnu.cuda-10.0.cudnn7.6.tar.gz) in our experiment.

3) install
```bash
tar -zxf TensorRT-7.0.0.11.Ubuntu-18.04.x86_64-gnu.cuda-10.0.cudnn7.6.tar.gz
vim ~/.bashrc

# Add codes in your file ~/.bashrc
export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:<your-TensorRT-lib-path>
# for example
# export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/data/zpzhang/TensorRT-7.0.0.11/lib

source ~/.bashrc
conda activate fastOcean
cd TensorRT-7.0.0.11/python
# Remember declare your python version=3.7
pip install tensorrt-7.0.0.11-cp37-none-linux_x86_64.whl
cd TensorRT-7.0.0.11/graphsurgeon
pip install graphsurgeon-0.4.1-py2.py3-none-any.whl
```

4) Verify the installation
```
python
import tensorrt
```

### Step3. Install Torch2trt
```
conda activate fastOcean
git clone https://github.com/NVIDIA-AI-IOT/torch2trt
cd torch2trt
python setup.py install
```
Verify the installation
```
python
import torch2trt
```

### Step4. Set up VOT2020 toolkit
step1 will automatically download vot toolkit.
```
conda activate fastOcean
mkdir vot-workspace
cd vot-workspace
vot workspace vot2020 --workspace <workspace-path>
cp $tracker_path/trackers.ini .
```
`<workspace-path>` indicates the absolute path of `vot-workspace`, e.g. `/data/zpzhang/vot-workspace`. `tracker_path` indicates the absolute path of our tracker, e.g. `/data/zpzahng/fastOcean`.

### Step5. Download the pre-trained model
Please download and unzip the pre-trained model [here](https://drive.google.com/open?id=1oY-8Qe5-QQIwEzh6V6MfyZGkTW8Esk-9) in `$tracker_path/`.

### Step6. Modify path in codes
- Modify `$tracker_path` of line7 in `<workspace-path>/trackers.ini`, e.g. `/data/zpzahng/fastOcean`.
- Modify `$tracker_path` of line130 in `$tracker_path/vot_fastOcean.py`, e.g. `/data/zpzahng/fastOcean`.

### Step7. Run testing
In `vot-workspace`,
```
conda activate fastOcean
CUDA_VISIBLE_DEVICES=$gpu_id vot evaluate --workspace <workspace-path> fastOcean
```
`$gpu_id` indicates the ID of GPU that you plan to run the tracker.


### Step8. Get results
In `vot-workspace`,
```
vot analysis --workspace <workspace-path> fastOcean --output json 
```

### Note
- Before running our tracker, you should consider temporally stop other programs that take much CPU or GPU resources. Otherwise, it may bring negative effect on realtime running. If you only run the baseline experiment, please ignore this. 
- We run our tracker with RTX 2080Ti. The GPU with worse capacity will degradate reatime results. 
- If you met the error `PermissionError: [Errno 13] Permission denied: '/tmp/torch_extensions/_prroi_pooling/lock'`, please remove `/tmp/torch_extensions/ _prroi_pooling` and rerun the tracker. If other user in your machine have compiled prroi pooling before, this may happens. Besides, if you have compiled pproi_pooling before, please remove `/tmp/torch_extensions/`. Otherwise, you may fail to compile in the new conda environment.
- If you met the error `ImportError: libcudart.so.10.0: cannot open shared object file: No such file or directory`, please run `sudo cp /usr/local/cuda-10.0/lib64/libcudart.so.10.0 /usr/local/lib/libcudart.so.10.0 && sudo ldconfig`.
